/*									tab:4
 * "Copyright (c) 2000-2003 The Regents of the University  of California.  
 * All rights reserved.
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose, without fee, and without written agreement is
 * hereby granted, provided that the above copyright notice, the following
 * two paragraphs and the author appear in all copies of this software.
 * 
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY FOR
 * DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT
 * OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF THE UNIVERSITY OF
 * CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS
 * ON AN "AS IS" BASIS, AND THE UNIVERSITY OF CALIFORNIA HAS NO OBLIGATION TO
 * PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS."
 *
 * Copyright (c) 2002-2007 Intel Corporation
 * All rights reserved.
 *
 * This file is distributed under the terms in the attached INTEL-LICENSE     
 * file. If you do not find these files, copies can be found by writing to
 * Intel Research Berkeley, 2150 Shattuck Avenue, Suite 1300, Berkeley, CA, 
 * 94704.  Attention:  Intel License Inquiry.
 */


/**
 * @author Brano Kusy
 *
 * Modified by Jimenez Yvan
 * Modified by Alvarez Emmanuel
 */

import java.lang.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.color.*;
import java.io.*;
import net.tinyos.message.*;
import net.tinyos.packet.*;
import net.tinyos.util.*;
import java.util.ArrayList;
import java.util.Vector;
import java.awt.Graphics;


public class CameraGUI implements MessageListener, Messenger 
{
    MoteIF mote; 		// For talking to the antitheft root node

    /* Various swing components we need to use after initialisation */
    JFrame frame;		// The whole frame
    JFrame picture1;		// Picture 1 frame
    JFrame picture2;		// Picture 2 frame
    JTextArea mssgArea;		// The message area

    /* The checkboxes for the requested settings */
    JCheckBox clearAfterSave, isColor, isVga, isCompression;
    /*The ComboBoxes for size and color settings*/
    JComboBox colorList, sizeList;
    /*Options values for size and color settings*/
    final String qvga="QVGA (320x240)", vga="VGA (640x480)", grayscale="Grayscale", rgb="RGB";
    /*Do we show the picture live?*/
    boolean showPLive = false;
    boolean showPAuto = false;
    /*The save dialog*/
    JFileChooser save_diag=new JFileChooser();
    //Jpanel
    Drawppmpgm draw,draw2;

    /** possible file extension */
    final String[] EXTENSION_ENTREE = {"ppm","pgm"};

    /* true if we want to stop the capture */
    boolean stop;
    
    public CameraGUI(boolean connect) 
    {
		try 
		{
		    guiInit();
		}
		catch(Exception e) 
		{
		    e.printStackTrace();
		    System.exit(2);
		}
		if (!connect)
		    return;
	
		try 
		{
		    /* Setup communication with the mote and request a messageReceived
		       callback when an AlertMsg is received */
		    System.out.print("connecting to serial forwarder: failed");
		    mote = new MoteIF(this);
		    System.out.println("\b\b\b\b\b\bok    ");
		    mote.registerListener(new Bigmsg_frame_partMsg(), this);
		    mote.registerListener(new ImgStatMsg(), this);
		}
		catch(Exception e) 
		{
		    e.printStackTrace();
		    System.exit(2);
		}
    }

    /* Build up the GUI using Swing magic. Nothing very exciting here - the
       BagPanel class makes the code a bit cleaner/easier to read. */
    private void guiInit() throws Exception 
	{

		/* Main panel */
		JPanel mainPanel = new JPanel(new BorderLayout());
		mainPanel.setMinimumSize(new Dimension(240, 600));
		mainPanel.setPreferredSize(new Dimension(240, 600));
		
		picture1 = new JFrame("Current Picture");
		picture2 = new JFrame("Last Picture");
		
		/*Calling the PPM/PGM reader with the created image format*/
		draw = new Drawppmpgm();
		draw2= new Drawppmpgm();

		picture1.add(draw);
		picture2.add(draw2);

		/* The message area */
		JScrollPane mssgPanel = new JScrollPane();
		mssgPanel.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
		mssgPanel.setAutoscrolls(true);
		mssgArea = new JTextArea();
		mssgArea.setFont(new java.awt.Font("Monospaced", Font.PLAIN, 20));
		mainPanel.add(mssgPanel, BorderLayout.SOUTH);
		mssgPanel.setPreferredSize(new Dimension(240,140));
		mssgPanel.getViewport().add(mssgArea, null);


		/* The button area */
		BagPanel buttonPanel = new BagPanel();
		GridBagConstraints c = buttonPanel.c;
		
		//Padding (GUI Look)
		c.ipady=30;

		c.fill = GridBagConstraints.HORIZONTAL;
		c.gridwidth = GridBagConstraints.REMAINDER;


		buttonPanel.makeLabel("OPTIONS:", JLabel.CENTER);


		/*COLOR COMBO BOX*/
		String[] colorChoice = {grayscale,rgb};
		/*Create the combo box*/
		colorList=new JComboBox(colorChoice);
		buttonPanel.add(colorList);


		/*SIZE COMBO BOX*/
		String[] sizeChoice = {qvga,vga};
		sizeList=new JComboBox(sizeChoice);
		buttonPanel.add(sizeList);


		c.fill = GridBagConstraints.HORIZONTAL;
		c.gridwidth = GridBagConstraints.REMAINDER;


		buttonPanel.makeSeparator(SwingConstants.HORIZONTAL);


		/*ONE-SHOT MODE*/
		buttonPanel.makeLabel("ONE-SHOT MODE:", JLabel.CENTER);


		/*CAPTURE BUTTON*/
		ActionListener getImageAction = new ActionListener() {
			public void actionPerformed(ActionEvent e) 
			{
				int s_x = 320;
				int s_y = 240;
				if (sizeList.getSelectedItem().toString().equals(vga))
				{   
				    s_x = 640;
				    s_y = 480;
				}
				picture1.setVisible(true);
				picture1.setSize(s_x,s_y);
				showLive();
			}
		};
		buttonPanel.makeButton("Capture", getImageAction);


		
		//Set the checkbox JPEG to false, compression Jpeg isn't working anyway..
		isCompression = new JCheckBox(); isCompression.setSelected(false);
		clearAfterSave = new JCheckBox(); clearAfterSave.setSelected(true);



		/*SAVE BUTTON, AND SAVE DIALOG*/
		ActionListener settingsAction = new ActionListener() {
			public void actionPerformed(ActionEvent e) {

			//Default filename (format: "picture_(current time)")
			save_diag.setSelectedFile(new File("picture_" + System.currentTimeMillis()));

			//We want a "Save file" type of file dialog
			int returnvalue = save_diag.showSaveDialog(null);


			if (returnvalue == JFileChooser.APPROVE_OPTION)
			{
				File file = save_diag.getSelectedFile();
				// We call the save() function with the path of the saved file
				//as a parameter
				save(file.getAbsolutePath());
			}
			else
			{
				System.out.println("File chooser cancel button clicked");
				System.out.println(sizeList.getSelectedItem().toString());
				System.out.println(colorList.getSelectedItem().toString());
			}
			return;
		    	
			}
		};
		buttonPanel.makeButton("Save", settingsAction);


		/*NON-STOP MODE*/

		buttonPanel.makeLabel("NON-STOP MODE:", JLabel.CENTER);

		/*Play*/
		ActionListener auto_play = new ActionListener() {
			public void actionPerformed(ActionEvent e) 
			{
				int s_x = 320;
				int s_y = 240;
				if (sizeList.getSelectedItem().toString().equals(vga))
				{   
				    s_x = 640;
				    s_y = 480;
				} 
				picture1.setVisible(true);
				picture1.setSize(s_x,s_y);
				picture2.setVisible(true);
				picture2.setSize(s_x,s_y);
				stop = false;
		    	getAuto();
			}
		};
		buttonPanel.makeButton("Play >>", auto_play);

		/*Stop*/
		ActionListener auto_stop = new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				System.out.println("STOP");
					stop=true;
			}
		};
		buttonPanel.makeButton("Stop", auto_stop);

		mainPanel.add(buttonPanel, BorderLayout.EAST);

		/* The frame part */
		frame = new JFrame("CameraGUI");
		frame.setSize(mainPanel.getPreferredSize());
		frame.getContentPane().add(mainPanel);
		frame.setVisible(true);
		frame.addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) { System.exit(0); }
	    	});
    }

    /* Add a message to the message area, auto-scroll to end */
    public synchronized void message(String s) 
	{
		mssgArea.append(s + "\n");
		mssgArea.setCaretPosition(mssgArea.getDocument().getLength());
    }

    /* Popup an error message */
    void error(String msg) 
	{
		JOptionPane.showMessageDialog(frame, msg, "Error", JOptionPane.ERROR_MESSAGE);
    }

    /* max colums*/
    int max_x;
    /* max rows */
    int max_y;
    /* current column */
    int curr_x=0;
    /* current row */
    int curr_y=0;
    
    ArrayList<short[]> data = new ArrayList<short[]>();
    Vector Points;
    
    long currentTime;
    
    public void showLive()
    {
    	showPLive = true;
    	curr_x = 1;
    	curr_y = 1;
    	draw.removeAll();
    	draw.updateUI();
	getImage();
    }
    
    public void getAuto()
    {
    	showPAuto = true;
	/* stop the capture? */
	if(stop == false)
	{
		draw2.removeAll();
		draw2.updateGraphics(draw.getG());
	    	showLive();
	}
    }
    
    public void getImage() 
    {
    	currentTime = System.currentTimeMillis();
		CmdMsg smsg = new CmdMsg();
		int cs = 0;
			
		/* Build and send command message */ 
		if (colorList.getSelectedItem().toString().equals(rgb))
		{
		    cs = cs | 0x01;
		}
		if (sizeList.getSelectedItem().toString().equals(vga))
		{
		    cs = cs | 0x02;
		    max_x = 640;
		    max_y = 480;
		} 
		else 
		{
		    max_x = 320;
		    max_y = 240;
		}
		if (isCompression.isSelected()) 
		{
		    cs = cs | 0x04;
		}
		smsg.set_cmd((short)cs);
		try 
		{
		    mote.send(MoteIF.TOS_BCAST_ADDR, smsg);
		}
		catch (IOException e) 
		{
		    error("Cannot send message to mote");
		}
    }

    public void save(String path) {
    	try {
	    String file_name;
	    file_name=path;
	    if (isCompression.isSelected())
		file_name += ".bytes";
	    else if (colorList.getSelectedItem().toString().equals(grayscale))
		file_name += ".pgm";
	    else
		file_name += ".ppm";
	    FileOutputStream fostr = null;
	    try {
		fostr = new FileOutputStream(file_name);
	    }
	    catch (Exception e){
		message("Can't write " + file_name + " file!\nMake sure the dir exists!\n");
		return;
	    };		
	
	    DataOutputStream stream = null;
	    OutputStreamWriter writer = null;
	    String pgmHeader =  "P2\r\n" + max_x + " " + max_y + "\r\n" + "255\r\n";
	    String ppmHeader =  "P3\r\n" + max_x + " " + max_y + "\r\n" + "255\r\n";
	    		
	    if (isCompression.isSelected()) {
		stream = new DataOutputStream(fostr);
	    } else {
		writer = new OutputStreamWriter(fostr);
		if (colorList.getSelectedItem().toString().equals(grayscale))
		    writer.write(pgmHeader);
		else
		    writer.write(ppmHeader);
	    }

	    int cnt_row = 1;
	    int red_byte = 0;
	    int blue_byte = 0;
	    int green_byte = 0;
	    int green_lo = 0;
	    int green_hi = 0;

	        boolean is_first_byte = true;
	        for (int i = 0; i < data.size(); i++) 
	        {
			    for (int j = 0; j < 64; j++) 
			    {
				int curr_byte = 0;
	
				if (j < data.get(i).length)
				    curr_byte = (data.get(i)[j] & 0xFF);
				else if (i == data.size() - 1)
				    break;
		    	    	
				if (isCompression.isSelected()) 
				{
				    stream.writeByte(curr_byte);
				} 
				else if (colorList.getSelectedItem().toString().equals(grayscale)) 
				{
				    writer.write((curr_byte & 0xFF) + " ");
				    if (++cnt_row >= max_x)
				    {
					cnt_row = 0;
					writer.write("\r\n");
				    }
				} 
				else 
				{
				    if (is_first_byte) 
				    {
					is_first_byte = false;
					red_byte = (curr_byte & 0x1F);
					green_lo = (curr_byte & 0xE0);
					green_lo = ((green_lo >> 5) & 0x07);
				    } 
				    else 
				    {
					blue_byte = ((curr_byte >> 3) & 0x1F);
					green_hi = (curr_byte & 0x07);
					green_hi = (green_hi << 3);
					green_byte = (green_lo | green_hi);
	
					red_byte <<= 3;
					blue_byte <<= 3;
					green_byte <<= 2;
					writer.write((red_byte & 0xFF) + " " + (green_byte&0xFF) + " " + (blue_byte&0xFF) + " ");
					if (++cnt_row >= max_x) 
					{
					    cnt_row = 0;
					    writer.write("\r\n");
					}
					is_first_byte = true;
				    }
				}
		    }
		}
    	if (isCompression.isSelected()){
	    stream.flush();
	    stream.close();
    	} else {
	    writer.flush();
	    writer.close();
    	}
		message("Saved " + file_name + "file and cleared data structures.\nReady for the next img.");
    	}
    	catch(Exception e){System.out.println(e);}
    	
        if (clearAfterSave.isSelected()){
	        data = new ArrayList<short[]>();
        }
    }
    
    public void updateGraphic(short[] tmp, int imgType)
    {
    	int red_byte = 0;
        int blue_byte = 0;
		int green_byte = 0;
		int green_lo = 0;
		int green_hi = 0;
		boolean first_byte=true;
		Points = new Vector();
		
		for (int xx=0; xx<tmp.length; xx++)
		{
		    if (curr_x == max_x)
		    {
		    	curr_x = 0;
		    	curr_y ++;
		    }
		        
		    int curr_byte=0;
		    curr_byte = (tmp[xx] & 0xFF);
			    
		    if (imgType == 2)
		    {
		        int b = (curr_byte & 0xFF);
		        Color cl = new Color(b,b,b);
   	            Points.add(new PointTrace ( curr_x , curr_y , cl ));
   	            curr_x++;
		    }
		    else
		    {
		        if (first_byte)
		        {
				    first_byte = false;
		  		    red_byte=(curr_byte & 0x1F);
				    green_lo=(curr_byte & 0xE0);
				    green_lo=((green_lo >> 5) & 0x07);
				}
				else
				{
				    blue_byte = ((curr_byte >> 3) & 0X1F);
				    green_hi = (curr_byte & 0x07);
				    green_hi = (green_hi << 3);
				    green_byte = (green_lo | green_hi);
				     
				    red_byte <<= 3;
				    blue_byte <<= 3;
				    green_byte <<= 2;
					    
				    int r,v,b;
				    r = (red_byte & 0xFF);
				    v = (green_byte & 0xFF);
				    b = (blue_byte & 0xFF);
					    
				    Color cl = new Color(r,v,b);
				    Points.add(new PointTrace ( curr_x , curr_y , cl ));
				    first_byte = true;
				    curr_x++;
				}
		    }
		}
		draw.updateGraphics(Points);		
    }
    
    /* Message received from mote network. Update message area if it's
       a theft message. */
    
    short []getMsgBuf(Bigmsg_frame_partMsg msg) {
    	int length = msg.dataLength() - Bigmsg_frame_partMsg.offset_buf(0);
        short[] tmp = new short[length];
        for (int index0 = 0; index0 < length; index0++)
            tmp[index0] = msg.getElement_buf(index0);
        return tmp;
    }
    
    public void messageReceived(int dest_addr, Message msg) 
    {
		if (msg instanceof Bigmsg_frame_partMsg) 
		{
		    Bigmsg_frame_partMsg fpMsg = (Bigmsg_frame_partMsg)msg;
		    
		    if (fpMsg.get_part_id() % 50 == 0)
		    	message("Msg ID " + fpMsg.get_part_id());
		    
		    while (fpMsg.get_part_id() >= data.size())
		    	data.add(data.size(),new short[0]);
		    
		    data.set(fpMsg.get_part_id(), getMsgBuf(fpMsg));
		    
		    if (showPLive)
		    {
		    	int imgType;
		    	if (colorList.getSelectedItem().toString().equals(grayscale))
				{ imgType = 2; }
				else
				{ imgType = 3; }
			/* update the current picture panel */	
		        updateGraphic(getMsgBuf(fpMsg),imgType);
		    }
		    if (showPAuto)
		    {
		    	if (curr_y > max_y)
		    	{
		    		getAuto();
		    	}
		    }
		} 
		else if (msg instanceof ImgStatMsg) 
		{
		    ImgStatMsg isMsg = (ImgStatMsg)msg;
		    message("Image capture stats:");
		    message("  img type: " + isMsg.get_type());
		    message("  img width: " + isMsg.get_width());
		    message("  img height: " + isMsg.get_height());
		    message("  img size: " + isMsg.get_data_size());
		}
    }
    
    public void load(String filename) 
    {
    	try 
    	{
		    BufferedReader r = new BufferedReader(new FileReader(filename));
		    String line = r.readLine();//header line 1
		    line = r.readLine(); //header line 2
		    line = r.readLine(); //header line 3
		    line = r.readLine(); //header line 4
		    line = r.readLine();
		    int i = 0, j = 0;
		    short[] values = new short[64];
		    while (line != null) 
		    {
				java.util.StringTokenizer t = new java.util.StringTokenizer(line);
				while (t.hasMoreTokens()) 
				{
				    String id_str = t.nextToken();
				    if (id_str.length() != 0) 
				    {
				    	values[i++] = Short.parseShort(id_str);
						if (i == 64) 
						{
						    Bigmsg_frame_partMsg fpMsg = new Bigmsg_frame_partMsg();
						    fpMsg.set_part_id(j);
						    fpMsg.set_buf(values);
						    this.messageReceived(0, fpMsg);
						    i = 0;
						    j++;
						    values = new short[64];
						}
				    }
				}
				line = r.readLine();
		    }
    	}
    	catch(Exception e) 
    	{
    		System.out.println("CameraGUI.load():" + e.toString());
    	}
    }

    /* Just start the app... */
    public static void main(String[] args) {
    	CameraGUI me = new CameraGUI(true);
    }
}
